@@ -67,6 +67,10 @@
  * user interface very often uses \r{su_uz} (i.e., \c{size_t}).
  * Other behaviour is explicitly declared with a "big" prefix, as in
  * "biglist", but none such object does exist at the time of this writing.
+ * }\li{
+ * \SU requires an eight (8) or more byte alignment on the stack and heap.
+ * This is because some of its facilities may use the lower (up to) three
+ * bits of pointers for internal, implementation purposes.
  * }}
  */
 
@@ -346,27 +350,8 @@ do{\
 #  endif
 # endif
 
-#elif defined __GNUC__ /* __clang__ */
-# undef su_CC_GCC
-# undef su_CC_VCHECK_GCC
-# define su_CC_GCC 1
-# define su_CC_VCHECK_GCC(X,Y) \
-      (__GNUC__ +0 > (X) || (__GNUC__ +0 == (X) && __GNUC_MINOR__ +0 >= (Y)))
-# define su_CC_EXTEN __extension__
-# define su_CC_PACKED __attribute__((packed))
-# if !defined su_CC_BOM &&\
-      defined __BYTE_ORDER__ && defined __ORDER_LITTLE_ENDIAN__ &&\
-      defined __ORDER_BIG_ENDIAN
-#  if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
-#   define su_CC_BOM su_CC_BOM_LITTLE
-#  elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
-#   define su_CC_BOM su_CC_BOM_BIG
-#  else
-#   error Unsupported __BYTE_ORDER__
-#  endif
-# endif
-
-#elif defined __PCC__ /* __GNUC__ */
+/* __GNUC__ after some other Unix compilers which also define __GNUC__ */
+#elif defined __PCC__ /* __clang__ */
 # undef su_CC_PCC
 # undef su_CC_VCHECK_PCC
 # define su_CC_PCC 1
@@ -397,6 +382,26 @@ do{\
 # define su_CC_EXTEN /* __extension__ (ignored) */
 # define su_CC_PACKED __attribute__((packed))
 
+#elif defined __GNUC__ /* __TINYC__ */
+# undef su_CC_GCC
+# undef su_CC_VCHECK_GCC
+# define su_CC_GCC 1
+# define su_CC_VCHECK_GCC(X,Y) \
+      (__GNUC__ +0 > (X) || (__GNUC__ +0 == (X) && __GNUC_MINOR__ +0 >= (Y)))
+# define su_CC_EXTEN __extension__
+# define su_CC_PACKED __attribute__((packed))
+# if !defined su_CC_BOM &&\
+      defined __BYTE_ORDER__ && defined __ORDER_LITTLE_ENDIAN__ &&\
+      defined __ORDER_BIG_ENDIAN
+#  if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+#   define su_CC_BOM su_CC_BOM_LITTLE
+#  elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+#   define su_CC_BOM su_CC_BOM_BIG
+#  else
+#   error Unsupported __BYTE_ORDER__
+#  endif
+# endif
+
 #elif !defined su_CC_IGNORE_UNKNOWN
 # error SU: This compiler is not yet supported.
 # error SU: To continue with your CFLAGS etc., define su_CC_IGNORE_UNKNOWN.
@@ -439,23 +444,33 @@ do{\
 # ifdef DOXYGEN
 #  define su_INLINE inline /*!< \_ */
 #  define su_SINLINE inline /*!< \_ */
-# elif su_CC_CLANG || su_CC_GCC || su_CC_PCC
+# elif su_CC_GCC
+   /* After lots of trouble with OpenBSD/gcc 4.2.1 and SunOS/gcc 3.4.3 */
+#  if !su_CC_VCHECK_GCC(3, 2) /* Unsure: only used C++ at that time */
+#   define su_INLINE extern __inline
+#   define su_SINLINE static __inline
+#  elif !su_CC_VCHECK_GCC(4, 3)
+#   define su_INLINE extern __inline __attribute__((always_inline))
+#   define su_SINLINE static __inline __attribute__((always_inline))
+   /* xxx gcc 8.3.0 bug: does not truly inline with -Os */
+#  elif !su_CC_VCHECK_GCC(8, 3) || !defined __OPTIMIZE__ ||\
+      !defined __STDC_VERSION__ || __STDC_VERSION__ +0 < 199901L
+#   define su_INLINE extern __inline __attribute__((gnu_inline))
+#   define su_SINLINE static __inline __attribute__((gnu_inline))
+#  elif !defined NDEBUG || !defined __OPTIMIZE__
+#   define su_INLINE static inline
+#   define su_SINLINE static inline
+#  else
+#   define su_INLINE inline
+#   define su_SINLINE static inline
+#  endif
+# elif su_CC_CLANG || su_CC_PCC
 #  if defined __STDC_VERSION__ && __STDC_VERSION__ +0 >= 199901L
-    /* That gcc is totally weird */
-#   if su_OS_OPENBSD && su_CC_GCC
-#    define su_INLINE extern __inline __attribute__((gnu_inline))
-#    define su_SINLINE static __inline __attribute__((gnu_inline))
-    /* All CCs coming here know __OPTIMIZE__ */
-#   elif !defined NDEBUG || !defined __OPTIMIZE__
+#   if !defined NDEBUG || !defined __OPTIMIZE__
 #    define su_INLINE static inline
 #    define su_SINLINE static inline
 #   else
-     /* xxx gcc 8.3.0 bug: does not truly inline with -Os */
-#    if su_CC_GCC && defined __OPTIMIZE_SIZE__
-#     define su_INLINE inline __attribute__((always_inline))
-#    else
-#     define su_INLINE inline
-#    endif
+#    define su_INLINE inline
 #    define su_SINLINE static inline
 #   endif
 #  else
@@ -499,16 +514,24 @@ do{\
 #endif
 
 /* Basic support macros, with side effects */
-/*! \_ */
+/*! Absolute value. */
 #define su_ABS(A) ((A) < 0 ? -(A) : (A))
-/*! \_ */
+/*! Cramp \a{X} to be in between \a{A} and \a{B}, inclusive. */
 #define su_CLIP(X,A,B) (((X) <= (A)) ? (A) : (((X) >= (B)) ? (B) : (X)))
-/*! \_ */
-#define su_MAX(A,B) ((A) < (B) ? (B) : (A))
-/*! \_ */
-#define su_MIN(A,B) ((A) < (B) ? (A) : (B))
-/*! \_ */
+/*! Is power of two? */
 #define su_IS_POW2(X) ((((X) - 1) & (X)) == 0)
+/*! Maximum value. */
+#define su_MAX(A,B) ((A) < (B) ? (B) : (A))
+/*! Minimum value. */
+#define su_MIN(A,B) ((A) < (B) ? (A) : (B))
+/*! Round down \a{X} to nearest multiple of \a{BASE}. */
+#define su_ROUND_DOWN(X,BASE) (((X) / (BASE)) * (BASE))
+/*! Ditto, if \a{BASE} is a power of two. */
+#define su_ROUND_DOWN2(X,BASE) ((X) & (~((BASE) - 1)))
+/*! Round up \a{X} to nearest multiple of \a{BASE}. */
+#define su_ROUND_UP(X,BASE) ((((X) + ((BASE) - 1)) / (BASE)) * (BASE))
+/*! Ditto, if \a{BASE} is a power of two. */
+#define su_ROUND_UP2(X,BASE) (((X) + ((BASE) - 1)) & (~((BASE) - 1)))
 
 /* Alignment.  Note: su_uz POW2 asserted in POD section below! */
 #if defined __STDC_VERSION__ && __STDC_VERSION__ +0 >= 201112L
@@ -516,26 +539,39 @@ do{\
 # define su_ALIGNOF(X) _Alignof(X)
 #else
    /*! \c{_Alignof()} if available, something hacky otherwise */
-# define su_ALIGNOF(X) ((sizeof(X) + sizeof(su_uz)) & ~(sizeof(su_uz) - 1))
+# define su_ALIGNOF(X) su_ROUND_UP2(sizeof(X), su__ZAL_L)
 #endif
 
+/*! Align a pointer \a{MEM} by the \r{su_ALIGNOF()} of \a{OTYPE}, and cast
+ * the result to \a{DTYPE}. */
+#define su_P_ALIGN(DTYPE,OTYPE,MEM) \
+   su_R(DTYPE,\
+      su_IS_POW2(su_ALIGNOF(OTYPE))\
+         ? su_ROUND_UP2(su_R(su_up,MEM), su_ALIGNOF(OTYPE))\
+         : su_ROUND_UP(su_R(su_up,MEM), su_ALIGNOF(OTYPE)))
+
 /* Roundup/align an integer;  Note: POW2 asserted in POD section below! */
 /*! Overalign an integer value to a size that cannot cause just any problem
- * for anything which does not use special alignment directives. */
-#define su_Z_ALIGN(X) ((su_S(su_uz,X) + 2*su__ZAL_L) & ~((2*su__ZAL_L) - 1))
+ * for anything which does not use special alignment directives.
+ * \remarks{It is safe to assume that \r{su_P_ALIGN()} can be used to place an
+ * object into a memory region spaced with it.} */
+#define su_Z_ALIGN_OVER(X) su_ROUND_UP2(su_S(su_uz,X), 2 * su__ZAL_L)
 
-/*! Smaller than \r{su_Z_ALIGN()}, but sufficient for basic plain-old-data. */
-#define su_Z_ALIGN_SMALL(X) ((su_S(su_uz,X) + su__ZAL_L) & ~(su__ZAL_L - 1))
+/*! Smaller than \r{su_Z_ALIGN_OVER()}, but sufficient for plain-old-data. */
+#define su_Z_ALIGN(X) su_ROUND_UP2(su_S(su_uz,X), su__ZAL_L)
 
 /*! \r{su_Z_ALIGN()}, but only for pointers and \r{su_uz}. */
-#define su_Z_ALIGN_PZ(X) ((su_S(su_uz,X) + su__ZAL_S) & ~(su__ZAL_S - 1))
+#define su_Z_ALIGN_PZ(X) su_ROUND_UP2(su_S(su_uz,X), su__ZAL_S)
 
-# define su__ZAL_S su_MAX(su_ALIGNOF(su_uz), su_ALIGNOF(void*))
-# define su__ZAL_L su_MAX(su__ZAL_S, su_ALIGNOF(su_u64))/* XXX FP,128bit */
+/* (These are below MCTA()d to be of equal size[, however].)
+ * _L must adhere to the minimum aligned claimed in the \mainpage */
+# define su__ZAL_S su_MAX(sizeof(su_uz), sizeof(void*))
+# define su__ZAL_L su_MAX(su__ZAL_S, sizeof(su_u64))/* XXX FP,128bit */
 
 /* Variants of ASSERT */
 #if defined NDEBUG || defined DOXYGEN
 # define su_ASSERT_INJ(X) /*!< Injection! */
+# define su_ASSERT_INJOR(X,Y) Y /*!< Injection! */
 # define su_ASSERT_NB(X) ((void)0) /*!< No block. */
 # define su_ASSERT(X) do{}while(0) /*!< \_ */
 # define su_ASSERT_LOC(X,FNAME,LNNO) do{}while(0) /*!< \_ */
@@ -553,6 +589,7 @@ do{\
 # define su_ASSERT_NYD_LOC(X,FNAME,LNNO) do{}while(0) /*!< \_ */
 #else
 # define su_ASSERT_INJ(X) X
+# define su_ASSERT_INJOR(X,Y) X
 
 # define su_ASSERT_NB(X) \
    su_R(void,((X) ? su_TRU1 \
@@ -608,17 +645,27 @@ do if(!(X)){\
 }while(0)
 #endif /* defined NDEBUG || defined DOXYGEN */
 
-/*! Create a bit mask for the bit range LO..HI -- HI cannot use highest bit! */
+/*! There are no bit-\c{enum}erations, but we use \c{enum}s as such, since the
+ * only other option for bit constants would be preprocessor macros.
+ * Since enumerations are expected to represent a single value, a normal
+ * integer is often used to store enumeration values.
+ * To be explicit, this macro is used instead, \a{X} is the pod, \a{Y} is the
+ * enumeration name that is actually meant. */
+#define su_BITENUM_IS(X,Y) X
+
+/*! Create a bit mask for the inclusive bit range \a{LO} to \a{HI}.
+ * \remarks{\a{HI} cannot use highest bit!}
+ * \remarks{Identical to \r{su_BITS_RANGE_MASK().} */
 #define su_BITENUM_MASK(LO,HI) (((1u << ((HI) + 1)) - 1) & ~((1u << (LO)) - 1))
 
-/* For injection macros like su_DBG(), NDBG, DBGOR, 64, 32, 6432 */
+/*! For injection macros like su_DBG(), NDBG, DBGOR, 64, 32, 6432 */
 #define su_COMMA ,
 
 /* Debug injections */
 #if defined su_HAVE_DEBUG && !defined NDEBUG
-# define su_DBG(X) X
-# define su_NDBG(X)
-# define su_DBGOR(X,Y) X
+# define su_DBG(X) X /*!< \_ */
+# define su_NDBG(X) /*!< \_ */
+# define su_DBGOR(X,Y) X /*!< \_ */
 #else
 # define su_DBG(X)
 # define su_NDBG(X) X
@@ -659,23 +706,25 @@ do{\
 #endif /* su_HAVE_DEVEL || su_HAVE_DEBUG */
 
 /* Development injections */
-#if defined su_HAVE_DEVEL || defined su_HAVE_DEBUG /* Not: !defined NDEBUG) */
-# define su_DVL(X) X
-# define su_NDVL(X)
-# define su_DVLOR(X,Y) X
+#if defined su_HAVE_DEVEL || defined su_HAVE_DEBUG /* Not: !defined NDEBUG) */\
+      || defined DOXYGEN
+# define su_DVL(X) X /*!< \_ */
+# define su_NDVL(X) /*!< \_ */
+# define su_DVLOR(X,Y) X /*!< \_ */
 #else
 # define su_DVL(X)
 # define su_NDVL(X) X
 # define su_DVLOR(X,Y) Y
 #endif
 
-/* To avoid files that are overall empty */
+/*! To avoid files that are overall empty */
 #define su_EMPTY_FILE() typedef int su_CONCAT(su_notempty_shall_b_, su_FILE);
 
 /* C field init */
-#if su_C_LANG && defined __STDC_VERSION__ && __STDC_VERSION__ +0 >= 199901L
-# define su_FIELD_INITN(N) .N =
-# define su_FIELD_INITI(I) [I] =
+#if (su_C_LANG && defined __STDC_VERSION__ && \
+      __STDC_VERSION__ +0 >= 199901L) || defined DOXYGEN
+# define su_FIELD_INITN(N) .N = /*!< \_ */
+# define su_FIELD_INITI(I) [I] = /*!< \_ */
 #else
 # define su_FIELD_INITN(N)
 # define su_FIELD_INITI(I)
@@ -687,7 +736,8 @@ do{\
    /*! The offset of field \a{F} in the type \a{T}. */
 # define su_FIELD_OFFSETOF(T,F) __builtin_offsetof(T, F)
 #else
-# define su_FIELD_OFFSETOF(T,F) su_S(su_uz,su_S(su_up,&(su_S(T *,su_NIL)->F)))
+# define su_FIELD_OFFSETOF(T,F) \
+      su_S(su_uz,su_S(su_up,&(su_R(T *,0x1)->F)) - 1)
 #endif
 
 /*! Distance in between the fields \a{S}tart and \a{E}end in type \a{T}. */
@@ -699,7 +749,7 @@ do{\
 
 /* Multithread injections */
 #ifdef su_HAVE_MT
-# define su_MT(X) X
+# define su_MT(X) X /*!< \_ */
 #else
 # define su_MT(X)
 #endif
@@ -719,17 +769,18 @@ do{\
 
 /* SMP injections */
 #ifdef su_HAVE_SMP
-# define su_SMP(X) X
+# define su_SMP(X) X /*!< \_ */
 #else
 # define su_SMP(X)
 #endif
 
 /* String stuff.
  * __STDC_VERSION__ is ISO C99, so also use __STDC__, which should work */
-#if defined __STDC__ || defined __STDC_VERSION__ || su_C_LANG
-# define su_STRING(X) #X
-# define su_XSTRING(X) su_STRING(X)
-# define su_CONCAT(S1,S2) su__CONCAT_1(S1, S2)
+#if defined __STDC__ || defined __STDC_VERSION__ || su_C_LANG || \
+      defined DOXYGEN
+# define su_STRING(X) #X /*!< \_ */
+# define su_XSTRING(X) su_STRING(X) /*!< \_ */
+# define su_CONCAT(S1,S2) su__CONCAT_1(S1, S2) /*!< \_ */
 # define su__CONCAT_1(S1,S2) S1 ## S2
 #else
 # define su_STRING(X) "X"
@@ -737,27 +788,30 @@ do{\
 # define su_CONCAT(S1,S2) S1/* will no work out though */S2
 #endif
 
-/* Compare (maybe mixed-signed) integers cases to T bits, unsigned,
- * T is one of our homebrew integers, e.g., UCMP(32, su_ABS(n), >, wleft).
- * Note: does not sign-extend correctly, that is still up to the caller */
-#if su_C_LANG
+#if su_C_LANG || defined DOXYGEN
+   /*! Compare (maybe mixed-signed) integers cases to \a{T} bits, unsigned,
+    * \a{T} is one of our homebrew integers, e.g.,
+    * \c{UCMP(32, su_ABS(n), >, wleft)}.
+    * \remarks{Does not sign-extend correctly, this is up to the caller.} */
 # define su_UCMP(T,A,C,B) (su_S(su_ ## u ## T,A) C su_S(su_ ## u ## T,B))
 #else
 # define su_UCMP(T,A,C,B) \
       (su_S(su_NSPC(su) u ## T,A) C su_S(su_NSPC(su) u ## T,B))
 #endif
 
-/* Casts-away (*NOT* cast-away) */
+/*! Casts-away (*NOT* cast-away) */
 #define su_UNCONST(T,P) su_R(T,su_R(su_up,su_S(void const*,P)))
+/*! Casts-away (*NOT* cast-away) */
 #define su_UNVOLATILE(T,P) su_R(T,su_R(su_up,su_S(void volatile*,P)))
-/* To avoid warnings with modern compilers for "char*i; *(s32_t*)i=;" */
+/*! To avoid warnings with modern compilers for "char*i; *(s32_t*)i=;" */
 #define su_UNALIGN(T,P) su_R(T,su_R(su_up,P))
 #define su_UNXXX(T,C,P) su_R(T,su_R(su_up,su_S(C,P)))
 
 /* Avoid "may be used uninitialized" warnings */
-#if defined NDEBUG && !(defined su_HAVE_DEBUG || defined su_HAVE_DEVEL)
-# define su_UNINIT(N,V) su_S(void,0)
-# define su_UNINIT_DECL(V)
+#if (defined NDEBUG && !(defined su_HAVE_DEBUG || defined su_HAVE_DEVEL)) || \
+      defined DOYGEN
+# define su_UNINIT(N,V) su_S(void,0) /*!< \_ */
+# define su_UNINIT_DECL(V) /*!< \_ */
 #else
 # define su_UNINIT(N,V) N = V
 # define su_UNINIT_DECL(V) = V
@@ -766,9 +820,11 @@ do{\
 /*! Avoid "unused" warnings */
 #define su_UNUSED(X) ((void)(X))
 
-/* Variable-type size (with byte array at end) */
-#if su_C_LANG && defined __STDC_VERSION__ && __STDC_VERSION__ +0 >= 199901L
+#if (su_C_LANG && defined __STDC_VERSION__ && \
+      __STDC_VERSION__ +0 >= 199901L) || defined DOXYGEN
+   /*! Variable-type size (with byte array at end) */
 # define su_VFIELD_SIZE(X)
+   /*! Variable-type size (with byte array at end) */
 # define su_VSTRUCT_SIZEOF(T,F) sizeof(T)
 #else
 # define su_VFIELD_SIZE(X) \
@@ -1305,8 +1361,8 @@ INLINE u32 su_state_get(void){
 /*! Interaction with the SU library (global) state machine:
  * test whether all (not any) of \a{flags} are set in \r{su_state_get()}. */
 INLINE boole su_state_has(uz flags){
-   uz f = flags & su__STATE_GLOBAL_MASK;
-   return ((su__state & f) == f);
+   flags &= su__STATE_GLOBAL_MASK;
+   return ((su__state & flags) == flags);
 }
 
 /*! \_ */
@@ -1335,9 +1391,10 @@ EXPORT s32 su_err_no(void);
 /*! \_ */
 EXPORT s32 su_err_set_no(s32 eno);
 
-/*! Return string(s) describing C error number eno.
- * This is effectively identical to \r{su_err_name()} if the compile-time
- * option \r{su_HAVE_DOCSTRINGS} is missing. */
+/*! Return string(s) describing C error number \a{eno}.
+ * Effectively identical to \r{su_err_name()} if either the compile-time
+ * option \r{su_HAVE_DOCSTRINGS} is missing (always), or when
+ *  \r{su_state_has()} \r{su_STATE_REPRODUCIBLE} set. */
 EXPORT char const *su_err_doc(s32 eno);
 
 /*! \_ */
@@ -1693,6 +1750,30 @@ inline T const &get_max(T const &a, T const &b) {return su_MAX(a, b);}
 template<class T>
 inline T const &get_min(T const &a, T const &b) {return su_MIN(a, b);}
 
+/*! \copydoc{su_ROUND_DOWN()} */
+template<class T>
+inline T const &get_round_down(T const &a, T const &b){
+   return su_ROUND_DOWN(a, b);
+}
+
+/*! \copydoc{su_ROUND_DOWN2()} */
+template<class T>
+inline T const &get_round_down2(T const &a, T const &b){
+   return su_ROUND_DOWN2(a, b);
+}
+
+/*! \copydoc{su_ROUND_UP()} */
+template<class T>
+inline T const &get_round_up(T const &a, T const &b){
+   return su_ROUND_UP(a, b);
+}
+
+/*! \copydoc{su_ROUND_UP2()} */
+template<class T>
+inline T const &get_round_up2(T const &a, T const &b){
+   return su_ROUND_UP2(a, b);
+}
+
 /*! \copydoc{su_IS_POW2()} */
 template<class T> inline int is_pow2(T const &a) {return su_IS_POW2(a);}
 
@@ -1726,8 +1807,8 @@ public:
    /*! \copydoc{su_err_number} */
    enum err_number{
 #ifdef DOXYGEN
-      err_none,      /*!< No error. */
-      err_notobacco  /*!< No such errno, fallback: no mapping exists. */
+      enone,      /*!< No error. */
+      enotobacco  /*!< No such errno, fallback: no mapping exists. */
 #else
       su__CXX_ERR_NUMBER_ENUM
 # undef su__CXX_ERR_NUMBER_ENUM
@@ -1898,6 +1979,11 @@ NSPC_END(su)
  * The C++ variants deduce many more things, and automatically, through
  * (specializations of) \r{type_traits}, \r{type_toolbox}, and
  * \r{auto_type_toolbox}.
+ *
+ * Because the C++ versions are template wrappers around their \c{void*} based
+ * C "supertypes", it is inefficient or even impossible to use \SU collections
+ * for plain-old-data; to overcome this restriction (some) specializations to
+ * work with POD exist.
  */
 
 /*!
