@@ -150,8 +150,9 @@ struct su__cs_dict_node{
 /* "The const is preserved logically" */
 EXPORT struct su__cs_dict_node *su__cs_dict_lookup(
       struct su_cs_dict const *self, char const *key, void *lookarg_or_nil);
+/* *lookarg_or_nil is always updated */
 EXPORT s32 su__cs_dict_insrep(struct su_cs_dict *self, char const *key,
-      void *value, boole replace);
+      void *value, up replace_and_view_or_nil);
 #if DVLOR(1, 0)
 EXPORT void su__cs_dict_stats(struct su_cs_dict const *self);
 #endif
@@ -400,7 +401,7 @@ INLINE boole su_cs_dict_view_is_valid(struct su_cs_dict_view const *self){
 }
 
 /*! \_ */
-INLINE struct su_cs_dict_view *su_cs_dict_view_invalidate(
+INLINE struct su_cs_dict_view *su_cs_dict_view_reset(
       struct su_cs_dict_view *self){
    ASSERT(self);
    self->csdv_node = NIL;
@@ -469,6 +470,26 @@ INLINE struct su_cs_dict_view *su_cs_dict_view_next(
 EXPORT boole su_cs_dict_view_find(struct su_cs_dict_view *self,
       char const *key);
 
+/*! See \r{su_cs_dict_insert()}.
+ * Upon success 0 is returned and \r{su_cs_dict_view_is_valid()} will be true.
+ * It is also true if -1 is returned because an existing \a{key} has not been
+ * updated. */
+INLINE s32 su_cs_dict_view_reset_insert(struct su_cs_dict_view *self,
+      char const *key, void *value){
+   ASSERT(self);
+   ASSERT_RET(key != NIL, 0);
+   return su__cs_dict_insrep(self->csdv_parent, key, value, FAL0 | R(up,self));
+}
+
+/*! See \r{su_cs_dict_replace()}.
+ * Upon success 0 or -1 is returned and \r{su_cs_dict_view_is_valid()}. */
+INLINE s32 su_cs_dict_view_reset_replace(struct su_cs_dict_view *self,
+      char const *key, void *value){
+   ASSERT(self);
+   ASSERT_RET(key != NIL, 0);
+   return su__cs_dict_insrep(self->csdv_parent, key, value, TRU1 | R(up,self));
+}
+
 /*! Remove the key/value tuple of a \r{su_cs_dict_view_is_valid()} view,
  * then move to the next valid position, if any. */
 EXPORT struct su_cs_dict_view *su_cs_dict_view_remove(
@@ -521,8 +542,8 @@ class cs_dict : private su_cs_dict{
          return su_cs_dict_view_parent(this) == su_cs_dict_view_parent(&t);
       }
       boole is_valid(void) const {return su_cs_dict_view_is_valid(this);}
-      gview &invalidate(void){
-         SELFTHIS_RET(su_cs_dict_view_invalidate(this));
+      gview &reset(void){
+         SELFTHIS_RET(su_cs_dict_view_reset(this));
       }
       char const *key(void) const {return su_cs_dict_view_key(this);}
       void *data(void) {return su_cs_dict_view_data(this);}
@@ -534,6 +555,12 @@ class cs_dict : private su_cs_dict{
       boole find(void const *key){
          return su_cs_dict_view_find(this, S(char const*,key));
       }
+      s32 reset_insert(void const *key, void *value){
+         return su_cs_dict_view_reset_insert(this, S(char const*,key), value);
+      }
+      s32 reset_replace(void const *key, void *value){
+         return su_cs_dict_view_reset_replace(this, S(char const*,key), value);
+      }
       gview &remove(void) {SELFTHIS_RET(su_cs_dict_view_remove(this));}
       sz cmp(gview const &t) const {return su_cs_dict_view_cmp(this, &t);}
    };
