@@ -59,7 +59,7 @@ struct su_mem_bag;
  * memory allocator (with \r{su_HAVE_MEM_BAG_AUTO}),
  * and/or a replacement for \c{alloca(3)} (\r{su_HAVE_MEM_BAG_LOFI}: a stack of
  * last-out, first-in memory storage).
- * In general the memory served is \r{su_Z_ALIGN_PZ()} aligned.
+ * In general the memory served is \r{su_Z_ALIGN()} aligned.
  * Allocation requests larger than about \r{su_S32_MAX} result in
  * \r{su_STATE_ERR_OVERFLOW} errors.
  *
@@ -169,10 +169,16 @@ EXPORT struct su_mem_bag *su_mem_bag_pop(struct su_mem_bag *self,
 
 /*! Get the bag that currently serves on the stack top.
  * Returns \SELF if there is no stack. */
-INLINE struct su_mem_bag *
-su_mem_bag_top(struct su_mem_bag *self){
+INLINE struct su_mem_bag *su_mem_bag_top(struct su_mem_bag *self){
    ASSERT_RET(self != NIL, NIL);
-   ASSERT_RET(self->mb_outer == NIL, su_mem_bag_top(self->mb_outer));
+#if ASSERT_INJOR(1, 0)
+   if(self->mb_outer != NIL){
+      ASSERT_EXEC(self->mb_outer == NIL, (void)0);
+      do
+         self = self->mb_outer;
+      while(self->mb_outer != NIL);
+   }
+#endif
    return (self->mb_top != NIL) ? self->mb_top : self;
 }
 
