@@ -118,8 +118,9 @@ encname(struct mailbox *mp, const char *name, int same, const char *box)
    } else {
       res = NULL;
 
-      if ((cachedir = ok_vlook(imap_cache)) == NULL ||
-            (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
+      if((cachedir = ok_vlook(imap_cache)) == NIL ||
+            (cachedir = fexpand(cachedir, (FEXP_NOPROTO | FEXP_LOCAL_FILE |
+               FEXP_NSHELL))) == NIL)
          goto jleave;
       eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
 
@@ -456,8 +457,9 @@ clean(struct mailbox *mp, struct cw *cw)
    FILE *fp = NULL;
    NYD_IN;
 
-   if ((cachedir = ok_vlook(imap_cache)) == NULL ||
-         (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
+   if((cachedir = ok_vlook(imap_cache)) == NIL ||
+         (cachedir = fexpand(cachedir, (FEXP_NOPROTO | FEXP_LOCAL_FILE |
+            FEXP_NSHELL))) == NIL)
       goto jleave;
    eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    if (su_cs_cmp_case(emailbox = mp->mb_imap_mailbox, "INBOX")) {
@@ -683,8 +685,9 @@ cache_list(struct mailbox *mp, const char *base, int strip, FILE *fp)
    enum okay rv = STOP;
    NYD_IN;
 
-   if ((cachedir = ok_vlook(imap_cache)) == NULL ||
-         (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
+   if((cachedir = ok_vlook(imap_cache)) == NIL ||
+         (cachedir = fexpand(cachedir, (FEXP_NOPROTO | FEXP_LOCAL_FILE |
+            FEXP_NSHELL))) == NIL)
       goto jleave;
    eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    name = n_autorec_alloc(namesz = su_cs_len(cachedir) +
@@ -842,8 +845,9 @@ cache_dequeue(struct mailbox *mp)
    enum okay rv = OKAY;
    NYD_IN;
 
-   if ((cachedir = ok_vlook(imap_cache)) == NULL ||
-         (cachedir = fexpand(cachedir, FEXP_LOCAL | FEXP_NOPROTO)) == NULL)
+   if((cachedir = ok_vlook(imap_cache)) == NIL ||
+         (cachedir = fexpand(cachedir, (FEXP_NOPROTO | FEXP_LOCAL_FILE |
+            FEXP_NSHELL))) == NIL)
       goto jleave;
    eaccount = mx_url_xenc(mp->mb_imap_account, TRU1);
    buf = n_autorec_alloc(bufsz = su_cs_len(cachedir) +
