@@ -178,23 +178,13 @@ FL struct str *
  * Routines that are not related to auto-reclaimed storage follow.
  */
 
-FL char *
-string_quote(char const *v) /* TODO too simpleminded (getrawlist(), +++ ..) */
-{
-   char const *cp;
-   uz i;
-   char c, *rv;
+FL boole
+n_is_maybe_regex_buf(char const *buf, uz len){
+   boole rv;
    NYD2_IN;
 
-   for (i = 0, cp = v; (c = *cp) != '\0'; ++i, ++cp)
-      if (c == '"' || c == '\\')
-         ++i;
-   rv = n_autorec_alloc(i +1);
-
-   for (i = 0, cp = v; (c = *cp) != '\0'; rv[i++] = c, ++cp)
-      if (c == '"' || c == '\\')
-         rv[i++] = '\\';
-   rv[i] = '\0';
+   rv = (su_cs_first_of_cbuf_cbuf(buf, len, "^[*+?|$", su_UZ_MAX
+         ) != su_UZ_MAX);
    NYD2_OU;
    return rv;
 }
