@@ -50,6 +50,7 @@ su_EMPTY_FILE()
 #include <su/cs.h>
 #include <su/mem.h>
 
+#include "mx/cmd.h"
 #include "mx/file-streams.h"
 #include "mx/net-socket.h"
 #include "mx/tty.h"
@@ -115,79 +116,106 @@ n_tls_verify_decide(void){
    return rv;
 }
 
-FL enum okay
-smime_split(FILE *ip, FILE **hp, FILE **bp, long xcount, int keep)
-{
-   struct myline {
-      struct myline  *ml_next;
-      uz         ml_len;
-      char           ml_buf[VFIELD_SIZE(0)];
+FL boole
+mx_smime_split(FILE *ip, FILE **hp, FILE **bp, long xcount, boole keep){
+   struct myline{
+      struct myline *ml_next;
+      uz ml_len;
+      char ml_buf[VFIELD_SIZE(0)];
    } *head, *tail;
-   char *buf;
-   uz bufsize, buflen, cnt;
    int c;
-   enum okay rv = STOP;
+   uz bufsize, buflen, cnt;
+   char *buf;
+   boole rv;
    NYD_IN;
 
+   rv = FAL0;
+   mx_fs_linepool_aquire(&buf, &bufsize);
+   *hp = *bp = NIL;
+
    if((*hp = mx_fs_tmp_open("smimeh", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
             mx_FS_O_REGISTER), NIL)) == NIL)
-      goto jetmp;
-   if((*bp = mx_fs_tmp_open("smimeb", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
-            mx_FS_O_REGISTER), NIL)) == NIL){
-      mx_fs_close(*hp);
-jetmp:
-      n_perr(_("tempfile"), 0);
       goto jleave;
-   }
+   if((*bp = mx_fs_tmp_open("smimeb", (mx_FS_O_RDWR | mx_FS_O_UNLINK |
+            mx_FS_O_REGISTER), NIL)) == NIL)
+      goto jleave;
 
-   head = tail = NULL;
-   buf = n_alloc(bufsize = LINESIZE);
+   head = tail = NIL;
    cnt = (xcount < 0) ? fsize(ip) : xcount;
 
-   while (fgetline(&buf, &bufsize, &cnt, &buflen, ip, 0) != NULL &&
-         *buf != '\n') {
-      if (!su_cs_cmp_case_n(buf, "content-", 8)) {
-         if (keep)
-            fputs("X-Encoded-", *hp);
-         for (;;) {
-            struct myline *ml = n_alloc(VSTRUCT_SIZEOF(struct myline, ml_buf
-                  ) + buflen +1);
-            if (tail != NULL)
+   for(;;){
+      if(fgetline(&buf, &bufsize, &cnt, &buflen, ip, FAL0) == NIL){
+         if(ferror(ip))
+            goto jleave;
+         break;
+      }
+      if(*buf == '\n')
+         break;
+
+      if(!su_cs_cmp_case_n(buf, "content-", 8)){
+         if(keep && fputs("X-Encoded-", *hp) == EOF)
+            goto jleave;
+         for(;;){
+            struct myline *ml;
+
+            ml = n_lofi_alloc(VSTRUCT_SIZEOF(struct myline, ml_buf) +
+                  buflen +1);
+            if(tail != NIL)
                tail->ml_next = ml;
             else
                head = ml;
             tail = ml;
-            ml->ml_next = NULL;
+            ml->ml_next = NIL;
             ml->ml_len = buflen;
             su_mem_copy(ml->ml_buf, buf, buflen +1);
-            if (keep)
-               fwrite(buf, sizeof *buf, buflen, *hp);
-            c = getc(ip);
-            ungetc(c, ip);
-            if (!su_cs_is_blank(c))
+            if(keep && fwrite(buf, sizeof *buf, buflen, *hp) != buflen)
+               goto jleave;
+            if((c = getc(ip)) == EOF || ungetc(c, ip) == EOF)
+               goto jleave;
+            if(!su_cs_is_blank(c))
                break;
-            fgetline(&buf, &bufsize, &cnt, &buflen, ip, 0);
+            if(fgetline(&buf, &bufsize, &cnt, &buflen, ip, FAL0) == NIL &&
+                  ferror(ip))
+               goto jleave;
          }
          continue;
       }
-      fwrite(buf, sizeof *buf, buflen, *hp);
+
+      if(fwrite(buf, sizeof *buf, buflen, *hp) != buflen)
+         goto jleave;
    }
    fflush_rewind(*hp);
 
-   while (head != NULL) {
-      fwrite(head->ml_buf, sizeof *head->ml_buf, head->ml_len, *bp);
+   while(head != NIL){
+      if(fwrite(head->ml_buf, sizeof *head->ml_buf, head->ml_len, *bp
+            ) != head->ml_len)
+         goto jleave;
       tail = head;
       head = head->ml_next;
-      n_free(tail);
+      n_lofi_free(tail);
    }
-   putc('\n', *bp);
-   while (fgetline(&buf, &bufsize, &cnt, &buflen, ip, 0) != NULL)
-      fwrite(buf, sizeof *buf, buflen, *bp);
-   fflush_rewind(*bp);
 
-   n_free(buf);
+   if(putc('\n', *bp) == EOF)
+      goto jleave;
+   while(fgetline(&buf, &bufsize, &cnt, &buflen, ip, FAL0) != NIL){
+      if(fwrite(buf, sizeof *buf, buflen, *bp) != buflen)
+         goto jleave;
+   }
+   fflush_rewind(*bp);
+   if(ferror(ip))
+      goto jleave;
+
    rv = OKAY;
 jleave:
+   if(!rv){
+      if(*bp != NIL)
+         mx_fs_close(*bp);
+      if(*hp != NIL)
+         mx_fs_close(*hp);
+   }
+
+   mx_fs_linepool_release(buf, bufsize);
+
    NYD_OU;
    return rv;
 }
@@ -224,8 +252,15 @@ smime_sign_assemble(FILE *hp, FILE *bp, FILE *tsp, char const *message_digest)
    fprintf(op, "\n--%s\n", boundary);
    fputs("Content-Type: application/pkcs7-signature; name=\"smime.p7s\"\n"
       "Content-Transfer-Encoding: base64\n"
-      "Content-Disposition: attachment; filename=\"smime.p7s\"\n"
-      "Content-Description: S/MIME digital signature\n\n", op);
+      "Content-Disposition: attachment; filename=\"smime.p7s\"\n", op);
+   /* C99 */{
+      char const *cp;
+
+      if(*(cp = ok_vlook(content_description_smime_signature)) != '\0')
+         fprintf(op, "Content-Description: %s\n", cp);
+   }
+   putc('\n', op);
+
    while ((c = getc(tsp)) != EOF) {
       if (c == '-') {
          while ((c = getc(tsp)) != EOF && c != '\n');
@@ -275,8 +310,15 @@ smime_encrypt_assemble(FILE *hp, FILE *yp)
 
    fputs("Content-Type: application/pkcs7-mime; name=\"smime.p7m\"\n"
       "Content-Transfer-Encoding: base64\n"
-      "Content-Disposition: attachment; filename=\"smime.p7m\"\n"
-      "Content-Description: S/MIME encrypted message\n\n", op);
+      "Content-Disposition: attachment; filename=\"smime.p7m\"\n", op);
+   /* C99 */{
+      char const *cp;
+
+      if(*(cp = ok_vlook(content_description_smime_message)) != '\0')
+         fprintf(op, "Content-Description: %s\n", cp);
+   }
+   putc('\n', op);
+
    while ((c = getc(yp)) != EOF) {
       if (c == '-') {
          while ((c = getc(yp)) != EOF && c != '\n');
@@ -302,89 +344,105 @@ jleave:
 }
 
 FL struct message *
-smime_decrypt_assemble(struct message *m, FILE *hp, FILE *bp)
-{
-   u32 lastnl = 0;
-   int binary = 0;
-   char *buf = NULL;
-   uz bufsize = 0, buflen, cnt;
-   long lns = 0, octets = 0;
-   struct message *x;
+mx_smime_decrypt_assemble(struct message *mp, FILE *hp, FILE *bp){
+   boole binary, lastnl;
+   long lns, octets;
    off_t offset;
+   uz bufsize, buflen, cnt;
+   char *buf;
+   struct message *xmp;
    NYD_IN;
 
-   x = n_autorec_alloc(sizeof *x);
-   *x = *m;
+   xmp = NIL;
+   mx_fs_linepool_aquire(&buf, &bufsize);
+
    fflush(mb.mb_otf);
    fseek(mb.mb_otf, 0L, SEEK_END);
    offset = ftell(mb.mb_otf);
+   lns = octets = 0;
+   binary = FAL0;
 
    cnt = fsize(hp);
-   while (fgetline(&buf, &bufsize, &cnt, &buflen, hp, 0) != NULL) {
+   while(fgetline(&buf, &bufsize, &cnt, &buflen, hp, FAL0) != NIL){
       char const *cp;
-      if (buf[0] == '\n')
+
+      if(buf[0] == '\n')
          break;
-      if ((cp = n_header_get_field(buf, "content-transfer-encoding", su_NIL)
-            ) != NULL)
-         if (!su_cs_cmp_case_n(cp, "binary", 7))
-            binary = 1;
-      fwrite(buf, sizeof *buf, buflen, mb.mb_otf);
+      if((cp = n_header_get_field(buf, "content-transfer-encoding", NIL)
+            ) != NIL)
+         if(!su_cs_cmp_case_n(cp, "binary", 7))
+            binary = TRU1;
+      if(fwrite(buf, sizeof *buf, buflen, mb.mb_otf) != buflen)
+         goto jleave;
       octets += buflen;
       ++lns;
    }
+   if(ferror(hp))
+      goto jleave;
 
-   {  struct time_current save = time_current;
+   /* C99 */{
+      struct time_current save;
+      int w;
+
+      save = time_current;
       time_current_update(&time_current, TRU1);
-      octets += mkdate(mb.mb_otf, "X-Decoding-Date");
+      if((w = mkdate(mb.mb_otf, "X-Decoding-Date")) == -1)
+         goto jleave;
+      octets += w;
       time_current = save;
    }
    ++lns;
 
    cnt = fsize(bp);
-   while (fgetline(&buf, &bufsize, &cnt, &buflen, bp, 0) != NULL) {
+   lastnl = FAL0;
+   while(fgetline(&buf, &bufsize, &cnt, &buflen, bp, FAL0) != NIL){
       lns++;
-      if (!binary && buf[buflen - 1] == '\n' && buf[buflen - 2] == '\r')
+      if(!binary && buf[buflen - 1] == '\n' && buf[buflen - 2] == '\r')
          buf[--buflen - 1] = '\n';
       fwrite(buf, sizeof *buf, buflen, mb.mb_otf);
       octets += buflen;
-      if (buf[0] == '\n')
-         ++lastnl;
-      else if (buf[buflen - 1] == '\n')
-         lastnl = 1;
+      if(buf[0] == '\n')
+         lastnl = lastnl ? TRUM1 : TRU1;
       else
-         lastnl = 0;
+         lastnl = (buf[buflen - 1] == '\n');
    }
+   if(ferror(bp))
+      goto jleave;
 
-   while (!binary && lastnl < 2) {
-      putc('\n', mb.mb_otf);
-      ++lns;
-      ++octets;
-      ++lastnl;
+   if(!binary && lastnl != TRUM1){
+      for(;;){
+         if(putc('\n', mb.mb_otf) == EOF)
+            goto jleave;
+         ++lns;
+         ++octets;
+         if(lastnl)
+            break;
+         lastnl = TRU1;
+      }
    }
 
-   mx_fs_close(hp);
-   mx_fs_close(bp);
-   n_free(buf);
-
    fflush(mb.mb_otf);
-   if (ferror(mb.mb_otf)) {
-      n_perr(_("decrypted output data"), 0);
-      x = NULL;
-   }else{
-      x->m_size = x->m_xsize = octets;
-      x->m_lines = x->m_xlines = lns;
-      x->m_block = mailx_blockof(offset);
-      x->m_offset = mailx_offsetof(offset);
+   if(!ferror(mb.mb_otf)){
+      xmp = n_autorec_alloc(sizeof *xmp);
+      *xmp = *mp;
+      xmp->m_size = xmp->m_xsize = octets;
+      xmp->m_lines = xmp->m_xlines = lns;
+      xmp->m_block = mailx_blockof(offset);
+      xmp->m_offset = mailx_offsetof(offset);
    }
+
+jleave:
+   mx_fs_linepool_release(buf, bufsize);
+
    NYD_OU;
-   return x;
+   return xmp;
 }
 
 FL int
 c_certsave(void *vp){
    FILE *fp;
    int *msgvec, *ip;
-   struct n_cmd_arg_ctx *cacp;
+   struct mx_cmd_arg_ctx *cacp;
    NYD_IN;
 
    cacp = vp;
@@ -395,9 +453,9 @@ c_certsave(void *vp){
       char *file, *cp;
 
       file = cacp->cac_arg->ca_next->ca_arg.ca_str.s;
-      if((cp = fexpand(file, FEXP_LOCAL_FILE | FEXP_NOPROTO)) == NULL ||
-            *cp == '\0'){
-         n_err(_("`certsave': file expansion failed: %s\n"),
+      if((cp = fexpand(file, (FEXP_NOPROTO | FEXP_LOCAL_FILE | FEXP_NSHELL))
+            ) == NIL || *cp == '\0'){
+         n_err(_("certsave: file expansion failed: %s\n"),
             n_shexp_quote_cp(file, FAL0));
          vp = NULL;
          goto jleave;
@@ -441,7 +499,12 @@ n_tls_rfc2595_hostname_match(char const *host, char const *pattern){
 
 FL int
 c_tls(void *vp){
+#ifdef mx_HAVE_NET
+   struct mx_socket so;
+   struct mx_url url;
+#endif
    uz i;
+   enum {a_FPRINT, a_CERTIFICATE, a_CERTCHAIN} mode;
    char const **argv, *varname, *varres, *cp;
    NYD_IN;
 
@@ -452,31 +515,51 @@ c_tls(void *vp){
 
    if((cp = argv[0])[0] == '\0')
       goto jesubcmd;
-   else if(su_cs_starts_with_case("fingerprint", cp)){
-#ifndef mx_HAVE_NET
-      n_err(_("`tls': fingerprint: no +sockets in *features*\n"));
-      n_pstate_err_no = su_ERR_OPNOTSUPP;
-      goto jleave;
-#else
-      struct mx_socket so;
-      struct mx_url url;
+   else if(su_cs_starts_with_case("fingerprint", cp))
+      mode = a_FPRINT;
+   else if(su_cs_starts_with_case("certificate", cp))
+      mode = a_CERTIFICATE;
+   else if(su_cs_starts_with_case("certchain", cp))
+      mode = a_CERTCHAIN;
+   else
+      goto jesubcmd;
 
-      if(argv[1] == NULL || argv[2] != NULL)
-         goto jesynopsis;
-      if((i = su_cs_len(*++argv)) >= U32_MAX)
-         goto jeoverflow; /* TODO generic for ALL commands!! */
-      if(!mx_url_parse(&url, CPROTO_CERTINFO, *argv))
-         goto jeinval;
-      if(!mx_socket_open(&so, &url)){ /* auto-close 4 CPROTO_CERTINFO if ok */
-         n_pstate_err_no = su_err_no();
-         goto jleave;
-      }
-      if(so.s_tls_finger == NULL)
+#ifndef mx_HAVE_NET
+   n_err(_("tls: fingerprint: no +sockets in *features*\n"));
+   n_pstate_err_no = su_ERR_OPNOTSUPP;
+   goto jleave;
+#else
+   if(argv[1] == NULL || argv[2] != NULL)
+      goto jesynopsis;
+   if((i = su_cs_len(*++argv)) >= U32_MAX)
+      goto jeoverflow; /* TODO generic for ALL commands!! */
+   if(!mx_url_parse(&url, CPROTO_CERTINFO, *argv))
+      goto jeinval;
+
+   if(!mx_socket_open(&so, &url)){
+      n_pstate_err_no = su_err_no();
+      goto jleave;
+   }
+   mx_socket_close(&so);
+
+   switch(mode){
+   case a_FPRINT:
+      if(so.s_tls_finger == NIL)
          goto jeinval;
       varres = so.s_tls_finger;
+      break;
+   case a_CERTIFICATE:
+      if(so.s_tls_certificate == NIL)
+         goto jeinval;
+      varres = so.s_tls_certificate;
+      break;
+   case a_CERTCHAIN:
+      if(so.s_tls_certchain == NIL)
+         goto jeinval;
+      varres = so.s_tls_certchain;
+      break;
+   }
 #endif /* mx_HAVE_NET */
-   }else
-      goto jesubcmd;
 
    n_pstate_err_no = su_ERR_NONE;
    vp = (char*)-1;
@@ -494,15 +577,15 @@ jleave:
    return (vp == NULL);
 
 jeoverflow:
-   n_err(_("`tls': string length or offset overflows datatype\n"));
+   n_err(_("tls: string length or offset overflows datatype\n"));
    n_pstate_err_no = su_ERR_OVERFLOW;
    goto jleave;
 
 jesubcmd:
-   n_err(_("`tls': invalid subcommand: %s\n"),
+   n_err(_("tls: invalid subcommand: %s\n"),
       n_shexp_quote_cp(*argv, FAL0));
 jesynopsis:
-   n_err(_("Synopsis: tls: <command> [<:argument:>]\n"));
+   mx_cmd_print_synopsis(mx_cmd_firstfit("tls"), NIL);
 jeinval:
    n_pstate_err_no = su_ERR_INVAL;
    goto jleave;
